﻿//using FLY.FeedbackRenZiJia.Client;
//using FLY.FeedbackRenZiJia.Common;
using LiveCharts;
using LiveCharts.Defaults;
using LiveCharts.Wpf;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using FLY.HeatingHelper;
using System.Threading.Tasks;

namespace FLY.HeatingHelper.UI.UIModule
{
    /// <summary>
    /// Page_HeatDetector.xaml 的交互逻辑
    /// </summary>
    public partial class Page_HeatDetector : Page,INotifyPropertyChanged
    {
        //private HeatBufServiceClient mHeatBuf;
        ViewModel_HeatDetector _vm;
        
        public Page_HeatDetector()
        {
            InitializeComponent();
        }

        public void InitData(object dat)
        {
            _vm = ViewModel_HeatDetector.Instance;
            _vm.SetData(dat as IThickHeatData);
            this.DataContext = _vm;
            uc_airRingShift.Init(dat);
            uc_heatBoltAnalyst.Init(dat);
        }

        private void button_back_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                NavigationService.GoBack();
            }
            catch
            {
                Application.Current.Shutdown();
            }
        }

        private void MyChart_MouseDown(object sender, MouseButtonEventArgs e)
        {
            
        }

        private void MyChart_MouseUp(object sender, MouseButtonEventArgs e)
        {
            clickUpPoint = e.GetPosition(myChart);
            IsClickDown = false;
            int startx, endx;
            if (clickDownPoint.X < clickUpPoint.X)
            {
                startx = (int)myChart.ConvertToChartValues(clickDownPoint).X;
                endx = (int)myChart.ConvertToChartValues(clickUpPoint).X;
            }
            else
            {
                endx = (int)myChart.ConvertToChartValues(clickDownPoint).X;
                startx = (int)myChart.ConvertToChartValues(clickUpPoint).X;
            }
            _vm.DataFrom = _vm.Xaxis[startx];
            _vm.DataTo = _vm.Xaxis[endx];
            _vm.IsDefault = false;
        }

        private void MyChart_MouseMove(object sender, MouseEventArgs e)
        {
            if (IsClickDown)
            {
                clickUpPoint = e.GetPosition(myChart);
                var dpoint = myChart.ConvertToChartValues(clickDownPoint);
                var upoint = myChart.ConvertToChartValues(clickUpPoint);
                _vm.RW = Math.Abs(clickUpPoint.X - clickDownPoint.X);
                _vm.RH = Math.Abs(clickUpPoint.Y - clickDownPoint.Y);
                if (clickUpPoint.X > clickDownPoint.X)
                {
                    _vm.RX = dpoint.X;
                }
                else
                {
                    _vm.RX = upoint.X;
                }
                if (clickUpPoint.Y > clickDownPoint.Y)
                {
                    _vm.RY = dpoint.Y;
                }
                else
                {
                    _vm.RY = upoint.Y;
                }
            }
        }

        private bool IsClickDown = false;
        private Point clickDownPoint;
        private Point clickUpPoint;
        private void MyChart_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            clickDownPoint = e.GetPosition(myChart);
            IsClickDown = true;
            _vm.RW = 0;
            _vm.RH = 0;
            //RX = point.X;
            //RY = point.Y;
            //ve1.X = 200;
            //ve1.Y = 30;
            //RX = 200;
            //RY = 30;
            //DataTo = DateTime.Now;
        }

        /// <summary>
        /// 由选定的Cluster计算Rect（RX，RY，RW，RH）
        /// </summary>
        public void Calculate_From_Class_To_Rxx()
        {
            int from = _vm._data.GetIndexFromID(_vm.Selected_Idx_From),
                to = _vm._data.GetIndexFromID(_vm.Selected_Idx_To);

            _vm.RX = from;
            _vm.RY = myChart.AxisY.First().ActualMaxValue;
            _vm.RW = myChart.ConvertToPixels(new Point(to, _vm.RY)).X - myChart.ConvertToPixels(new Point(from, _vm.RY)).X;
            _vm.RH = Math.Abs(myChart.ConvertToPixels(new Point(to, myChart.AxisY.First().ActualMaxValue)).Y
                - myChart.ConvertToPixels(new Point(to, myChart.AxisY.First().ActualMinValue)).Y);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void Default_cb_Click(object sender, RoutedEventArgs e)
        {
            if (_vm.IsDefault)
            {
                _vm.ResetSelectClass();
                Calculate_From_Class_To_Rxx();
            }
        }

        /// <summary>
        /// 按钮“下一步”
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            tc1.SelectedIndex++;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            _vm.SelectClass_Step(1);
            Calculate_From_Class_To_Rxx();
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            _vm.SelectClass_Step(-1);
            Calculate_From_Class_To_Rxx();
        }

        private async void Page_Loaded(object sender, RoutedEventArgs e)
        {
            await Task.Delay(200);
            Default_cb_Click(this, null);
        }

        private void Page_Unloaded(object sender, RoutedEventArgs e)
        {
            _vm.Clear();
            _vm._data.EndUse();
        }
    }

    public class ViewModel_HeatDetector : INotifyPropertyChanged
    {
        private static ViewModel_HeatDetector _instance = new ViewModel_HeatDetector();
        public IThickHeatData _data;

        static ViewModel_HeatDetector() { }
        private ViewModel_HeatDetector() { }

        public static ViewModel_HeatDetector Instance
        {
            get
            {
                return _instance;
            }
        }

        public void SetData(IThickHeatData dat)
        {
            if (_data == dat) return;
            Clear();
            _data = dat;
            dat.PropertyChanged += Data_Updated;
            ResetSelectClass();
            Update_Tab1();
        }

        public void Clear()
        {
            if (_data != null)
            {
                _data.PropertyChanged -= Data_Updated;
            }
        }
        private void Data_Updated(object o, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsDataReady")
            {
                if (_data.IsDataReady)
                {
                    Update_Tab1();
                }
            }
        }
        #region ViewModel For Page1
        public DateTime DBFrom { get; internal set; }
        public DateTime DBTo { get; internal set; }

        /// <summary>
        /// 选择要处理的数据（按时间）
        /// </summary>
        public DateTime DataFrom { get; set; }
        public DateTime DataTo { get; set; }

        /// <summary>
        /// 选择要处理的数据（按ID）
        /// </summary>
        public int Selected_Idx_From { get; set; }
        public int Selected_Idx_To { get; set; }

        public bool IsDefault { get; set; } = true;

        public ChartValues<ObservableValue> MyValues { get; set; }
        public ChartValues<double> MyValues2 { get; set; }
        public SeriesCollection MySeries { get; set; }

        public Func<double, string> Formatter { get; set; }

        public double RX { get; set; }
        public double RY { get; set; }
        public double RW { get; set; }
        public double RH { get; set; }
        #endregion

        private int _selected_class;
        public List<DateTime> Xaxis = new List<DateTime>();

        public void ResetSelectClass()
        {
            int from = 0, to = 0;
            //int a = _data.CalculateFromToByClass(-1, ref from, ref to);
            if ((_selected_class = _data.CalculateFromToByClass(-99, ref from, ref to)) >= 0)
            {
                Selected_Idx_From = from;
                Selected_Idx_To = to;
                var tmp = _data.GetDateTimeByID(from);
                DataFrom = (DateTime)tmp;
                tmp = _data.GetDateTimeByID(to);
                DataTo = (DateTime)tmp;
                Update_Tab1();
            }
        }

        public void SelectClass_Step(int step)
        {
            int from = 0, to = 0;
            int _sclass = _selected_class;
            while (true)
            {
                if ((_sclass = _data.CalculateFromToByClass(_sclass + step, ref from, ref to)) >= 0)
                {
                    if (from == to) continue;
                    _selected_class = _sclass;
                    Selected_Idx_From = from;
                    Selected_Idx_To = to;
                    var tmp = _data.GetDateTimeByID(from);
                    DataFrom = (DateTime)tmp;
                    tmp = _data.GetDateTimeByID(to);
                    DataTo = (DateTime)tmp;
                    return;
                }
                break;
            }
        }

        public void Update_Tab1()
        {
            if (!_data.IsDataReady) return;
            DBFrom = _data.TotalDataFrom;
            DBTo = _data.TotalDataTo;
            //DataFrom = _data.Dat_Times.First();
            //DataTo = _data.Dat_Times.Last();
            MyValues = new ChartValues<ObservableValue>();

            MyValues2 = new ChartValues<double>();

            Xaxis.Clear();
            int i, cnt = _data.Dat_Times.Count();
            for (i = 0; i < cnt; i++)
            {
                MyValues.Add(new ObservableValue((double)_data.Dat_Means[i]));
                MyValues2.Add((double)(_data.Dat_Sigmas[i] * 100 / _data.Dat_Means[i]));
                Xaxis.Add(_data.Dat_Times[i]);
            }

            var s1 = new LineSeries
            {
                Title = "厚度均值",
                Values = MyValues,
                StrokeThickness = 2,
                Fill = Brushes.Transparent,
                PointGeometrySize = 4,
                DataLabels = false,
                ScalesYAt = 0
            };
            var s2 = new LineSeries
            {
                Title = "Sigma*2",
                Values = MyValues2,
                StrokeThickness = 2,
                Fill = Brushes.Transparent,
                PointGeometrySize = 4,
                DataLabels = false,
                ScalesYAt = 1
            };
            MySeries = new SeriesCollection();
            MySeries.Add(s1);
            MySeries.Add(s2);

            Formatter = d =>
            {
                if ((d > Xaxis.Count()) || (d < 0))
                {
                    return d.ToString();
                }
                else
                {
                    return Xaxis[(int)d].ToString("MM-dd HH:mm:ss");
                }
            };
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class VMBase_HeatDetector:INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void Loaded(object s)
        {
            var tmpVM = ViewModel_HeatDetector.Instance;
            tmpVM.PropertyChanged += DataSelecter_PropertyChanged;
        }

        public void UnLoaded(object s)
        {
            var tmpVM = ViewModel_HeatDetector.Instance;
            tmpVM.PropertyChanged -= DataSelecter_PropertyChanged;
        }

        protected int MinFrame { get; set; }
        protected int MaxFrame { get; set; }

        public void DataSelecter_PropertyChanged(object s, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Selected_Idx_From")
            {
                MinFrame = (s as ViewModel_HeatDetector).Selected_Idx_From;
            }
            if (e.PropertyName == "Selected_Idx_To")
            {
                MaxFrame = (s as ViewModel_HeatDetector).Selected_Idx_To;
            }
        }
    }
}
