﻿using LiveCharts;
using LiveCharts.Wpf;
using MathNet.Numerics.LinearAlgebra;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;

namespace FLY.HeatingHelper.UI.UIModule
{
    public class UC_FrameDiffViewModel : INotifyPropertyChanged
    {
        IThickHeatData mdata;

        public UC_FrameDiffViewModel()
        {
        }
        /// <summary>
        /// 两个ID
        /// </summary>
        public int FrameIdx1 { get; set; } = -1;
        public int FrameIdx2 { get; set; } = -1;
        public SeriesCollection MySeries { get; set; } = new SeriesCollection();

        public void Init(IThickHeatData dat)
        {
            mdata = dat;
            PropertyChanged += DataSelecter_PropertyChanged;
        }

        Vector<double> thick1;
        Vector<double> thick2;
        Vector<double> heat1;
        Vector<double> heat2;
        Vector<double> thickDiff;
        Vector<double> heatDiff;

        void Update()
        {
            if (FrameIdx1 < 0 || FrameIdx2 < 0 || (mdata is null))
            {
                return;
            }
            thick1 = Vector<double>.Build.DenseOfArray(mdata.GetThicks(FrameIdx1, -1, -1));
            thick2 = Vector<double>.Build.DenseOfArray(mdata.GetThicks(FrameIdx2, -1, -1));
            heat1 = Vector<double>.Build.DenseOfArray(mdata.GetHeats(FrameIdx1, -1, -1));
            heat2 = Vector<double>.Build.DenseOfArray(mdata.GetHeats(FrameIdx2, -1, -1));
            thickDiff = thick2 - thick1;
            heatDiff = heat1 - heat2;
            UpdateSeries();
        }

        private ChartValues<double> v1 = new ChartValues<double>();
        private ChartValues<double> v2 = new ChartValues<double>();

        private void UpdateSeries()
        {
            if (MySeries.Count == 0)
            {
                var L1 = new LineSeries
                {
                    Title = "厚度变化",
                    Values = v1,
                    DataLabels = false,
                    Fill = new SolidColorBrush(Colors.Transparent),
                    PointGeometrySize = 3,
                    ScalesYAt = 0,
                };
                var L2 = new LineSeries
                {
                    Title = "加热率变化",
                    Values = v2,
                    DataLabels = false,
                    Fill = new SolidColorBrush(Colors.Transparent),
                    PointGeometrySize = 3,
                    ScalesYAt = 1,
                };
                MySeries.Add(L1);
                MySeries.Add(L2);
            }
            v1.Clear(); v1.AddRange(thickDiff);
            v2.Clear(); v2.AddRange(heatDiff);
        }

        private void DataSelecter_PropertyChanged(object s, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "FrameIdx1" || e.PropertyName == "FrameIdx2")
            {
                Update();
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
