﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.HeatingHelper.UI.UIModule
{


    class UC_HeatBoltAnalystViewModel : INotifyPropertyChanged
    {
        IThickHeatData mdata;
        public UC_FramePickerViewModel vm1 { get; set; } = new UC_FramePickerViewModel();
        public UC_FramePickerViewModel vm2 { get; set; } = new UC_FramePickerViewModel();
        public UC_FrameDiffViewModel vmDiff { get; set; } = new UC_FrameDiffViewModel();
        public UC_HeatBoltAnalystViewModel(IThickHeatData thickHeatData)
        {
            mdata = thickHeatData;
        }

        /// <summary>
        /// 最大幅间隔
        /// </summary>
        public int MaxFrameInterval { get; set; } = 5;
        /// <summary>
        /// 加热量变化数，可以是连续几个加热棒总的变化量
        /// </summary>
        public int HeatRateVar { get; set; } = 100;
        /// <summary>
        /// 太小的加热变化量需要忽略掉
        /// </summary>
        public int NeglectHeatRate { get; set; } = 5;
        /// <summary>
        /// 特征加热量的旁没有别的加热
        /// </summary>
        public int Separation { get; set; } = 5;

        public int SelectItem { get; set; } = 0;

        private bool isFilter = true;
        public bool IsFilter
        {
            get
            {
                return isFilter;
            }
            set
            {
                if(isFilter != value)
                {
                    isFilter = value;
                    if (isFilter) HeatBoltItems = filteredHeatBoltsItems;
                    else HeatBoltItems = heatBoltsItems;
                }
            }
        }

        public List<HeatBoltAnalystItem> HeatBoltItems { get; set; }

        private List<HeatBoltAnalystItem> heatBoltsItems;
        private List<HeatBoltAnalystItem> filteredHeatBoltsItems;

        public void SearchFeaturedBoltsItems(int startIdx,int endIdx)
        {
             heatBoltsItems = mdata.SearchFeaturedBoltsItem(startIdx, 
                endIdx,
                int.MaxValue,
                MaxFrameInterval, 
                HeatRateVar, 
                NeglectHeatRate, 
                Separation,5);
            filteredHeatBoltsItems = mdata.FilterFeaturedBoltsItem(heatBoltsItems, 0.9);
            if (IsFilter) HeatBoltItems = filteredHeatBoltsItems;
            else HeatBoltItems = heatBoltsItems;
            if (HeatBoltItems.Count > 0)
            {
                SelectItem = 0;
            }
        }

        public void Loaded(object s)
        {
            var tmpVM = ViewModel_HeatDetector.Instance;
            tmpVM.PropertyChanged += DataSelecter_PropertyChanged;
            vm1.InitData(mdata, tmpVM.Selected_Idx_From, tmpVM.Selected_Idx_To);
            vm2.InitData(mdata, tmpVM.Selected_Idx_From, tmpVM.Selected_Idx_To);
            vmDiff.Init(mdata);
            vm1.UpdateChart();
            vm2.UpdateChart();
        }

        public void UnLoaded(object s)
        {
            var tmpVM = ViewModel_HeatDetector.Instance;
            tmpVM.PropertyChanged -= DataSelecter_PropertyChanged;
        }

        private void DataSelecter_PropertyChanged(object s, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Selected_Idx_From")
            {
                vm1.MinFrame = (s as ViewModel_HeatDetector).Selected_Idx_From;
                vm2.MinFrame = vm1.MinFrame;
            }
            if (e.PropertyName == "Selected_Idx_To")
            {
                vm1.MaxFrame = (s as ViewModel_HeatDetector).Selected_Idx_To;
                vm2.MaxFrame = vm1.MaxFrame;
            }
        }

        public void Wrap_SearchFeaturedBoltsItems()
        {
            SearchFeaturedBoltsItems(vm1.MinFrame, vm1.MaxFrame);
            //SearchFeaturedBoltsItems(282, 284);
        }

        public void SetFrameIdx1(int idx)
        {
            vm1.SelectedFrame = vmDiff.FrameIdx1 = idx;
        }

        public void SetFrameIdx2(int idx)
        {
            vm2.SelectedFrame = vmDiff.FrameIdx2 = idx;
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
