﻿using MathNet.Numerics.LinearAlgebra;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FLY.HeatingHelper
{
    public abstract class ThickHeatData : IThickHeatData
    {
        readonly static object _sync = new object();    // 用于数据使用时的同步
        private bool lockTaken = false;
        protected string DBName;
        private BackgroundWorker worker = new BackgroundWorker();

        public ThickHeatData()
        {
            DBName = null;
        }

        public void SetDB(string dbname)
        {
            if (DBName == dbname) return;
            DBName = dbname;
            StartDBTask();
            //worker.WorkerSupportsCancellation = true;
            //worker.WorkerReportsProgress = true;
            //worker.DoWork += new DoWorkEventHandler((o, e) =>
            //{
            //    //Console.WriteLine("SetDB1 =" + Thread.CurrentThread.ManagedThreadId);
            //    ThreadPriority tmp = Thread.CurrentThread.Priority;
            //    Thread.CurrentThread.Priority = ThreadPriority.Lowest;
            //    Thread.Yield();
            //    lock (_sync)
            //    {
            //        ResetCluster();
            //        RefreshData();
            //    }
            //    Thread.CurrentThread.Priority = tmp;
            //});
            //worker.RunWorkerCompleted += (o, e) =>
            //{
            //    IsDataReady = true;
            //    StartDBTask();
            //};
            //worker.RunWorkerAsync();
            //Task.Factory.StartNew(() =>
            //{
            //    //    Console.WriteLine("SetDB1 =" + Thread.CurrentThread.ManagedThreadId);
            //    //while (true) ;

            //    ResetCluster();
            //        RefreshData();
            //});
            //Thread th = new Thread(() => 
            //{
            //    Thread.CurrentThread.Priority = ThreadPriority.Lowest;
            //    ResetCluster();
            //    RefreshData();
            //});
            //th.Start();
        }

        #region 数据库接口
        /// <summary>
        /// 从数据库中获取数据，清除以前数据
        /// </summary>
        public abstract void GetDataFromDB(bool reload = false);

        /// <summary>
        /// 从dataTable中提取数据到内部缓冲区
        /// </summary>
        protected abstract void FetchData();

        /// <summary>
        /// 从dataTable中提取新数据到内部缓冲区
        /// </summary>
        protected abstract int FetchNewData();
        #endregion

        #region 本地数据
        private DataTable dataTable = new DataTable();
        /// <summary>
        /// 数据缓冲区，按数据顺序存放的
        /// </summary>
        public List<int> _ids = new List<int>();
        public List<DateTime> _dat_times = new List<DateTime>();
        public List<double> _thick_means = new List<double>();
        public List<double> _thick_2sigmas = new List<double>();
        //protected List<List<double>> _thicks = new List<List<double>>();
        //protected List<List<double>> _heats = new List<List<double>>();
        protected List<double[]> _thicks = new List<double[]>();
        protected List<double[]> _heats = new List<double[]>();
        protected List<int> _resetBolts = new List<int>();    // 每幅数据的复位分区号
        protected List<double> _rotAngles = new List<double>();   // 每幅数据旋转角度
        public int BoltCnt { get; set; } = -1;
        protected int _index_from;
        protected int _index_to;
        protected int _last_cluster_idx;  // 最后聚类的点的索引号，当新数据到来时，可以只处理新数据了。
        #endregion

        #region IThickHeatData接口
        public bool IsDataReady { get; internal set; } = false;
        public DateTime DataTo { get; set; }
        public TimeSpan DataSpan { get; set; }
        public DateTime DFrom { get; set; }
        public DateTime DTo { get; set; }

        public int AirRingShift { get; internal set; } = 0;

        public bool BeginUse(bool isAsync = true)
        {
            if(isAsync)
            {
                return Monitor.TryEnter(_sync);
            }
            Monitor.Enter(_sync);
            return true;
        }
        public void EndUse()
        {
            Monitor.Exit(_sync);
        }

        public bool NewDataArrived { set => RefreshData(); }

        public DateTime TotalDataFrom { get; protected set; }

        public DateTime TotalDataTo { get; protected set; }

        public void CalculateAirRingShift()
        {
            //int[] idxs = CalculateMaxHeatVariation(thickClasses[0]);
            //double[][] VarDat = CalculateHeatThickVariation(idxs[0], idxs[1]);
            //double[] correlVec = CalculateCorrelVector(VarDat[0], VarDat[1]);
            //AirRingShift = CalculateAirRingShiftFromCorelVector(correlVec).Item1;
            if (AutoDetectEvents is null) return;
            var re = AutoDetectEvents.Find((e) => e.EvtCode == ThickHeatEventCode.EVC_RingShift);
            if (re is null) return;
            MaxSimilarityResult ms = re.EvtData as MaxSimilarityResult;
            AirRingShift = ms.frameShift;
        }

        public Tuple<int, double> CalculateAirRingShiftFromCorelVector(double[] correlVec)
        {
            double max = double.MinValue;
            int shift = 0;
            for (int j = 0; j < correlVec.Count(); j++)
            {
                if (double.IsNaN(correlVec[j])) continue;
                if (max < correlVec[j])
                {
                    max = correlVec[j];
                    shift = j;
                }
            }
            return new Tuple<int, double>(shift, max);
        }

        #region 根据厚度进行聚类
        /// <summary>
        /// 根据厚度均值进行聚类的类的描述
        /// </summary>
        public class ThickClass
        {
            public double center;            // 聚类的中心
            public List<int> points_idx;     // 聚类中的点，按时间顺序排列
            public int from;                 // 该类的起止点
            public int to;
            public ThickClass()
            {
                from = int.MaxValue;
                to = int.MinValue;
                points_idx = new List<int>();
            }
            public ThickClass(int idx, double mean)
            {
                from = to = idx;
                center = mean;
                points_idx = new List<int>();
                points_idx.Add(idx);
            }
            public void AddPoint(int idx, double mean)
            {
                if (points_idx.Contains(idx))
                {
                    return;
                }
                points_idx.Add(idx);
                if (from > idx)
                {
                    from = idx;
                }
                if (to < idx)
                {
                    to = idx;
                }
                int cnt = points_idx.Count();
                if (cnt == 1)
                {
                    center = mean;
                }
                else
                {
                    center = (center * (cnt - 1) + mean) / cnt;
                }
            }
            public void Merge(ThickClass tc)
            {
                int cnt1 = points_idx.Count();
                int cnt2 = tc.points_idx.Count();
                center = (center * cnt1 + tc.center * cnt2) / (cnt1 + cnt2);
                List<int> idxs = new List<int>();
                int i, j;
                i = 0; j = 0;
                while ((i < cnt1) && (j < cnt2))
                {
                    if (points_idx[i] < tc.points_idx[j])
                    {
                        idxs.Add(points_idx[i++]);
                    }
                    else
                    {
                        idxs.Add(tc.points_idx[j++]);
                    }
                }
                while (i < cnt1)
                {
                    idxs.Add(points_idx[i++]);
                }
                while (j < cnt2)
                {
                    idxs.Add(tc.points_idx[j++]);
                }
                points_idx = idxs;
                from = idxs.First();
                to = idxs.Last();
            }
        }

        /// <summary>
        /// thickClasses是类的列表，按最近归到该类的点的先后顺序排列
        /// </summary>
        private List<ThickClass> thickClasses = new List<ThickClass>();
        private double thred = 1.5;
        private double merge_thred = 1; // 考虑 merge_thred = thred * 70%
        private int dist_thred = 2;     // 类距离近，判断是否可以合拼

        /// <summary>
        /// 初始化聚类数据，类清空。
        /// </summary>
        private void ResetCluster()
        {
            _last_cluster_idx = -1;
            thickClasses.Clear();
        }

        /// <summary>
        /// 依据厚度均值进行聚类
        /// </summary>
        private void ClusterOnThickMean()
        {
            int i;

            for (i = _last_cluster_idx + 1; i < _thick_means.Count(); i++)
            {
                ThickClass tmp_tc = null;
                foreach (ThickClass tc in thickClasses)
                {
                    if ((Math.Abs(tc.center - _thick_means[i]) < thred) && (i - tc.to < dist_thred))
                    {
                        tc.AddPoint(i, _thick_means[i]);
                        tmp_tc = tc;
                        break;
                    }
                }
                if (tmp_tc is null)     // 没有找到类的归属，新建一个类
                {
                    thickClasses.Insert(0, new ThickClass(i, _thick_means[i]));
                }
                else
                {       // 找到类的归属，把归属的类放在链表头，表示时间最近的类
                    thickClasses.Remove(tmp_tc);
                    thickClasses.Insert(0, tmp_tc);
                }
                if (thickClasses.Count() < 2) continue;
                while (IsClassClosedEnough(thickClasses[0], thickClasses[1]))
                {       // 最近的两个类足够接近，则合并这两个类
                    thickClasses[0].Merge(thickClasses[1]);
                    thickClasses.RemoveAt(1);
                }
            }
            _last_cluster_idx = i - 1;
        }

        /// <summary>
        /// 判断两个类是否足够接近
        /// </summary>
        /// <param name="thickClass1"></param>
        /// <param name="thickClass2"></param>
        /// <returns></returns>
        private bool IsClassClosedEnough(ThickClass thickClass1, ThickClass thickClass2)
        {
            if (Math.Abs(thickClass1.center - thickClass2.center) > merge_thred)
            {
                return false;
            }
            if (thickClass2.from - thickClass1.to > dist_thred)
            {
                return false;
            }
            if (thickClass1.from - thickClass2.to > dist_thred)
            {
                return false;
            }
            return true;
        }

        private ThickClass _selectClass;
        /// <summary>
        /// 选择用于计算的类（数据）。selected_class==-99，表示自动选择一个cluster,否则出错
        /// </summary>
        public int CalculateFromToByClass(int selected_class, ref int from, ref int to)
        {
            if (selected_class >= 0)
            {
                if (selected_class < thickClasses.Count())
                {
                    from = _ids[thickClasses[selected_class].from];
                    to = _ids[thickClasses[selected_class].to];
                    return selected_class;
                }
                return -1;
            }
            else if (selected_class == -99)
            {
                int i;
                for (i = 0; i < thickClasses.Count(); i++)
                {
                    if (thickClasses[i].points_idx.Count() >= 2)
                    {
                        from = _ids[thickClasses[i].from];
                        to = _ids[thickClasses[i].to];
                        return i;
                    }
                }
                return -1;
            }
            else
            {
                return -1;
            }
        }

        #endregion
        public int _select_frame_idx1;
        public int _select_frame_idx2;
        //public double[] _heatVariation;
        //public double[] _thickVariation;

        #region 一些辅助计算的函数
        private double HeatVariation(int idx1, int idx2)
        {
            double[] heats1 = _heats[idx1].ToArray();
            double[] heats2 = _heats[idx2].ToArray();
            int i, cnt = heats1.Count();
            if (cnt <= 0)
                return double.NaN;

            double sum = 0;
            for (i = 0; i < heats1.Count(); i++)
            {
                sum += Math.Pow(heats1[i] - heats2[i], 2);
            }
            return Math.Sqrt(sum / cnt);
        }

        /// <summary>
        /// 运用选出类（数据）的点计算最大加热变化。返回产生最大值的数据索引
        /// </summary>
        private int[] CalculateMaxHeatVariation(object _aclass)
        {
            ThickClass _theclass = _aclass as ThickClass;
            int cnt = _theclass.points_idx.Count();
            int i, j;
            double max = double.MinValue;
            int idx1 = -1, idx2 = -1;
            int[] ret = new int[2];
            for (i = 0; i < cnt; i++)
            {
                for (j = i + 1; j < cnt; j++)
                {
                    double tmp = HeatVariation(_theclass.points_idx[i], _theclass.points_idx[j]);
                    if (tmp > max)
                    {
                        max = tmp;
                        idx1 = _theclass.points_idx[i];
                        idx2 = _theclass.points_idx[j];
                    }
                }
            }
            ret[0] = idx1;
            ret[1] = idx2;
            return ret;
        }

        /// <summary>
        /// 计算两个指定数据的_heatVariation数组和_thickVariation数组
        /// </summary>
        /// <param name="idx1"></param>
        /// <param name="idx2"></param>
        public double[][] CalculateHeatThickVariation(int idx1, int idx2)
        {
            int cnt = _heats[0].Count();
            double[][] ret = new double[2][];
            ret[0] = new double[cnt];
            ret[1] = new double[cnt];
            int i;
            for (i = 0; i < cnt; i++)
            {
                ret[0][i] = _thicks[idx2][i] - _thicks[idx1][i];
                ret[1][i] = _heats[idx2][i] - _heats[idx1][i];
            }
            return ret;
        }

        /// <summary>
        /// 计算相关系数
        /// </summary>
        /// <param name="d1"></param>
        /// <param name="d2"></param>
        /// <returns></returns>
        private static double CalculateCorrel(double[] d1, double[] d2)
        {
            int cnt = d1.Count();
            double[] da1 = new double[cnt];
            double[] da2 = new double[cnt];
            int j = 0;
            for (int i = 0; i < cnt; i++)
            {
                if (double.IsNaN(d1[i]) || double.IsNaN(d2[i])) continue;
                da1[j] = d1[i];
                da2[j++] = d2[i];
            }
            if (j <= 0) return double.NaN;
            double tmpd = MathNet.Numerics.Statistics.Correlation.Pearson(da1.Take(j), da2.Take(j));
            return (tmpd is double.NaN) ? 0 : tmpd;
        }

        /// <summary>
        /// 计算两个frame的加热和厚度差的协相关向量。
        /// </summary>
        public double[] CalculateCorrelVector(double[] heats, double[] thicks)
        {
            int cnt = heats.Count();
            if (cnt != thicks.Count())
            {
                return null;
            }
            double[] result = new double[cnt];
            double[] tmpthicks = thicks.Concat(thicks).ToArray();
            for (int i = 0; i < cnt; i++)
            {
                result[i] = CalculateCorrel(heats, tmpthicks.Skip(i).Take(cnt).ToArray());
            }
            return result;
        }

        /// <summary>
        /// 用在追边吹膜测厚中不同的旋转角度造成的数据的缩放
        /// </summary>
        /// <param name="data">一幅数据</param>
        /// <param name="mid">中间位置，一般只复位分区号</param>
        /// <param name="z">缩放比例</param>
        /// <returns></returns>
        public static double[] ReSampleForChangeRotAngle(double[] data, int mid, double z)
        {
            if (Math.Abs(z - 1) < 0.001)
            {
                return data.Clone() as double[];
            }
            z = 1 / z;

            // 拼接data，使得数据两端都是NaN
            int cnt = data.Count();
            var tmpdata = data.Concat(data).ToArray();
            int n = cnt / 2;
            int i;
            int tmpmid = mid;
            if (mid < n)
            {
                mid += cnt;
            }
            for (i = 0; i < mid - n; i++) tmpdata[i] = double.NaN;
            for (i = mid + cnt - n; i < 2 * cnt; i++) tmpdata[i] = double.NaN;

            double[] _diff = new double[cnt*2];

            // 计算新数据开始位置
            double npos = mid - z * n;
            int ni = mid - n;
            double bin=0;
            double sum=0;

            // 计算新数据
            for (i = 0; i < cnt; i++)
            {
                sum = 0;
                bin = 0;
                if ((npos + z < mid - n) || (npos >= mid + cnt - n))
                {
                    _diff[ni++] = double.NaN;
                    npos += z;
                    continue;
                }
                int nn = (int)npos;
                if (npos + z < nn + 1)
                {
                    _diff[ni++] = tmpdata[nn];
                    npos += z;
                    continue;
                }
                else
                {
                    if (!double.IsNaN(tmpdata[nn]))
                    {
                        sum += tmpdata[nn] * (nn + 1 - npos);
                        bin += (nn + 1 - npos);
                    }
                    nn++;
                    while (nn + 1 < npos + z)
                    {
                        if (!double.IsNaN(tmpdata[nn]))
                        {
                            sum += tmpdata[nn];
                            bin += 1;
                        }
                        nn++;
                    }
                    if (!double.IsNaN(tmpdata[nn]))
                    {
                        sum += tmpdata[nn] * (npos + z - nn);
                        bin += (npos + z - nn);
                    }
                    if (bin > 0)
                    {
                        _diff[ni++] = sum / bin;
                    }
                    else
                    {
                        _diff[ni++] = double.NaN;
                    }
                    npos += z;
                }
            }
            if (mid == tmpmid)
            {
                return _diff.Skip(cnt).Take(mid - n).Concat(_diff.Skip(mid - n).Take(cnt - mid + n)).ToArray();
            }
            else
            {
                return _diff.Skip(cnt).Take(cnt - n + tmpmid).Concat(_diff.Skip(cnt - n + tmpmid).Take(n - tmpmid)).ToArray();
            }
        }

        public Tuple<int, double> MaximumAt(IEnumerable<double> dat)
        {
            int cnt = dat.Count();
            int idx = -1;
            double m = double.MinValue;
            for (int i = 0; i < cnt; i++)
            {
                if (double.IsNaN(dat.ElementAt(i))) continue;
                if (dat.ElementAt(i) > m)
                {
                    idx = i;
                    m = dat.ElementAt(i);
                }
            }
            return new Tuple<int, double>(idx, m);
        }

        /// <summary>
        /// 通过加热变化量和厚度变化量计算加热到厚度的比例系数
        /// 算法：
        ///     1.按照最大相关对齐加热变化量和厚度变化量
        ///     2.根据thicks值去掉无效数据
        ///     3.加热量滤波，小于5的全部置零。
        ///     4.平滑加热量（高斯平滑）
        ///     5.清零厚度变化量中对应加热量为零的厚度数据。
        ///     6.累计加热变化量和厚度变化量绝对值之和
        ///     7.计算出比例系数。
        /// </summary>
        /// <param name="heats">加热变化量</param>
        /// <param name="thicks">厚度变化量</param>
        /// <param name="thickmean">厚度均值</param>
        /// <returns></returns>
        private double CalculateHeatToThickFactor(double[] heats, double[] thicks, double thickmean)
        {
            int cnt = thicks.Count();
            if (cnt < 2) return double.NaN;
            //Step 1
            var tmp = CalculateCorrelVector(heats, thicks);
            var res = MaximumAt(tmp);
            if (res.Item1 < 0) return double.NaN;
            var tmpthicks = thicks.Concat(thicks).Skip(res.Item1).Take(cnt).ToArray();

            // Step 2
            List<double> t = new List<double>();
            List<double> h = new List<double>();
            int i;
            for (i = 0; i < cnt; i++)
            {
                if (double.IsNaN(tmpthicks[i])) continue;
                t.Add(tmpthicks[i]);
                h.Add(heats[i]);
            }
            // Step 3
            for (i = 0; i < t.Count; i++)
            {
                if (h[i] < 5) h[i] = 0;
            }
            // Step 4
            var hh = EffectFilter(h.ToArray());
            // step 5
            for (i = 0; i < hh.Count(); i++)
            {
                if (hh[i] < 0.001) t[i] = 0;
            }
            // step 6
            double sumt = 0, sumh = 0;
            for (i = 0; i < h.Count; i++)
            {
                sumt += Math.Abs(t[i]);
                sumh += Math.Abs(h[i]);
            }

            // Step 7
            return sumh / (sumt / thickmean);
        }

        protected double[] RemoveNaN(double[] dat)
        {
            List<double> d = new List<double>();
            foreach(double dd in dat)
            {
                if (double.IsNaN(dd)) continue;
                d.Add(dd);
            }
            return d.ToArray();
        }

        public int GetIndexFromID(int id)
        {
            int cnt = _ids.Count();
            for(int i = 0; i < cnt; i++)
            {
                if (_ids[i] == id) return i;
            }
            return -1;
        }
        #endregion

        #region 获取数据

        /// <summary>
        /// 获取某幅厚度数据的一部分
        /// </summary>
        /// <param name="idx"></param>
        /// <param name="from"></param>
        /// <param name="to"></param>
        /// <returns></returns>
        public double[] GetThicks(int idx, int from, int to)
        {
            int index = GetIndexFromID(idx);
            if (index < 0) return null;
            return GetThicksByIndex(index, from, to);
        }
        private double[] GetThicksByIndex(int idx,int from, int to)
        {
            int cnt = _thicks[0].Count();
            if ((to >= cnt) || (to < 0)) to = cnt - 1;
            if ((from < 0) || (from > to)) from = 0;
            return _thicks[idx].Skip(from).Take(to - from + 1).ToArray();
        }
        public double[] GetThicks(int idx, int newResetBolt, double newAngle)
        {
            double[] ret = GetThicks(idx, -1, -1);
            if (ret is null) return null;
            int index = GetIndexFromID(idx);
            if (!double.IsNaN(newAngle))
            {
                if (Math.Abs(newAngle - _rotAngles[index]) > 0.001)
                {
                    ret = ReSampleForChangeRotAngle(ret, _resetBolts[index], newAngle / _rotAngles[index]);
                }
            }
            if (newResetBolt != int.MaxValue)
            {
                int cnt = ret.Count();
                int s = newResetBolt - _resetBolts[index];
                while (s < 0) s += cnt;
                while (s >= cnt) s -= cnt;
                return ret.Concat(ret).Skip(cnt - s).Take(cnt).ToArray();
            }
            return ret;
        }
        private double[] GetThicksByIndex(int idx, int newResetBolt, double newAngle)
        {
            double[] ret = GetThicksByIndex(idx, -1, -1);
            if (ret is null) return null;
            int index = idx;
            if (!double.IsNaN(newAngle))
            {
                if (Math.Abs(newAngle - _rotAngles[index]) > 0.001)
                {
                    ret = ReSampleForChangeRotAngle(ret, _resetBolts[index], newAngle / _rotAngles[index]);
                }
            }
            if (newResetBolt != int.MaxValue)
            {
                int cnt = ret.Count();
                int s = newResetBolt - _resetBolts[index];
                while (s < 0) s += cnt;
                while (s >= cnt) s -= cnt;
                return ret.Concat(ret).Skip(cnt - s).Take(cnt).ToArray();
            }
            return ret;
        }

        /// <summary>
        /// 获取某幅加热数据的一部分
        /// </summary>
        /// <param name="idx"></param>
        /// <param name="from"></param>
        /// <param name="to"></param>
        /// <returns></returns>
        public double[] GetHeats(int idx, int from, int to)
        {
            int index = GetIndexFromID(idx);
            if (index < 0) return null;
            return GetHeatsByIndex(index, from, to);
        }
        private double[] GetHeatsByIndex(int idx, int from, int to)
        {
            int cnt = _heats[0].Count();
            if ((to >= cnt) || (to < 0)) to = cnt - 1;
            if ((from < 0) || (from > to)) from = 0;
            return _heats[idx].Skip(from).Take(to - from + 1).ToArray();
        }


        /// <summary>
        /// 获取某幅厚度数据的复位分区号
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public int GetResetBolt(int idx)
        {
            int index = GetIndexFromID(idx);
            if (index < 0) return -1;
            return GetResetBoltByIndex(index);
        }
        private int GetResetBoltByIndex(int idx)
        {
            return _resetBolts[idx];
        }

        /// <summary>
        /// 获取某幅厚度数据的旋转角度
        /// </summary>
        /// <param name="idx"></param>
        /// <returns></returns>
        public double GetRotAngle(int idx)
        {
            int index = GetIndexFromID(idx);
            if (index < 0) return -1;
            return GetRotAngleByIndex(index);
        }
        private double GetRotAngleByIndex(int idx)
        {
            return _rotAngles[idx];
        }

        public int NormalBolt(int bolt, int boltcnt)
        {
            if (boltcnt == int.MaxValue) boltcnt = BoltCnt;
            while (bolt >= boltcnt) bolt -= boltcnt;
            while (bolt < 0) bolt += boltcnt;
            return bolt;
        }

        public List<DateTime> Dat_Times { get { return _dat_times; } }
        public List<double> Dat_Sigmas { get { return _thick_2sigmas; } }
        public List<double> Dat_Means { get { return _thick_means; } }

        public int GetRelativeID(int id, int relative)
        {
            int idx = GetIndexFromID(id);
            if (idx < 0) return -1;
            idx += relative;
            if (idx < 0 || idx >= _ids.Count()) return -1;
            return _ids[idx];
        }
        public DateTime? GetDateTimeByID(int id)
        {
            int idx = GetIndexFromID(id);
            if (idx < 0) return null;
            return _dat_times[idx];
        }
        #endregion
        interface ISearchModel
        {
            int AddOneData(int hr);
        }
        private class DataModel : ISearchModel
        {
            int stage;
            int idx;
            int s1_fromIdx = -1, s1_toIdx;
            int s2_fromIdx = -1, s2_toIdx;
            int s3_fromIdx = -1, s3_toIdx;
            int s2_heatRateSum;
            int heatRate_Total;
            int separation;
            int boltCnt;
            public DataModel(int hr_total, int sepa, int boltCnt)
            {
                stage = 0;
                s2_heatRateSum = 0;
                heatRate_Total = hr_total;
                separation = sepa;
                idx = 0;
                this.boltCnt = boltCnt;
            }
            List<HeatBoltAnalystItem> result = new List<HeatBoltAnalystItem>();
            /// <summary>
            /// 增加一个数据，返回1表示一个HeatBoltAnalystItem产生
            ///               返回0表示没有
            ///               返回-1表示找完了，再找出来的和前面会有重复
            /// </summary>
            /// <param name="hr"></param>
            public int AddOneData(int hr)
            {
                switch (stage)
                {
                    case 0://开始
                        if (hr == 0)
                        {
                            stage = 1;
                            s1_fromIdx = idx;
                            s1_toIdx = idx;
                        }
                        break;
                    case 1:
                        if (hr != 0)
                        {
                            if (s1_toIdx - s1_fromIdx >= separation)
                            {
                                stage = 2;
                                s2_fromIdx = idx;
                                s2_toIdx = idx;
                                s2_heatRateSum = hr;
                            }
                            else
                            {
                                RestartStage();
                            }
                        }
                        else
                        {
                            s1_toIdx = idx;
                        }
                        break;
                    case 2:
                        if (hr == 0)
                        {
                            if (Math.Abs(s2_heatRateSum) >= heatRate_Total)
                            {
                                s3_fromIdx = s3_toIdx = idx;
                                stage = 3;
                            }
                            else
                            {
                                stage = 1;
                                s1_fromIdx = s1_toIdx = idx;
                            }
                        }
                        else
                        {
                            if ((s2_heatRateSum > 0) && (hr > 0) ||
                                (s2_heatRateSum < 0) && (hr < 0))
                            {
                                s2_heatRateSum += hr;
                                s2_toIdx = idx;
                            }
                            else
                            {
                                RestartStage();
                            }
                        }
                        break;
                    case 3:
                        if (hr == 0)
                        {
                            s3_toIdx = idx;
                            if (Math.Abs(s3_toIdx - s3_fromIdx) >= separation)
                            {
                                var tmp = new HeatBoltAnalystItem()
                                {
                                    StartBolt = s2_fromIdx >= boltCnt ? s2_fromIdx - boltCnt : s2_fromIdx,
                                    EndBolt = s2_toIdx >= boltCnt ? s2_toIdx - boltCnt : s2_toIdx
                                };
                                result.Add(tmp);
                                s1_fromIdx = s3_fromIdx;
                                s1_toIdx = s3_toIdx;
                                stage = 1;
                                idx++;
                                return 1;
                            }
                        }
                        else
                        {
                            RestartStage();
                        }
                        break;
                }
                idx++;
                if (idx >= boltCnt)
                {
                    if (stage == 0 || (stage >= 1 && s1_toIdx >= boltCnt + separation))
                    {
                        return -1;
                    }
                }
                return 0;
            }

            private void RestartStage()
            {
                stage = 0;
            }

            public void ResetAll()
            {
                idx = -separation;
                result.Clear();
                RestartStage();
            }

            public List<HeatBoltAnalystItem> GetResult()
            {
                return result;
            }
        }

        /// <summary>
        /// 加热效果模拟，缺省高斯函数，sigma=1.0
        /// </summary>
        public double[] HeatEffect { get; set; } = new double[]
        {
            0.086,0.343,0.571,0.343,0.086
        };

        private double[] EffectFilter(double[] dat)
        {
            double[] tmp = dat.Concat(dat).Concat(dat).ToArray();
            var cnt = dat.Count();
            var lim = cnt * 2;
            int j;
            var filtercnt = HeatEffect.Count();
            if (filtercnt % 2 == 0) return null;
            double[] result = new double[cnt];
            var shift = (filtercnt - 1) / 2;
            for (int i = cnt; i < lim; i++)
            {
                result[i - cnt] = 0;
                for (j = -shift; j <= shift; j++)
                {
                    result[i - cnt] += tmp[i + j] * HeatEffect[j + shift];
                }
            }
            return result;
        }

        /// <summary>
        /// 搜索可以成为内标的加热特征
        /// </summary>
        /// <param name="startIdx"></param>
        /// <param name="endIdx"></param>
        /// <param name="searchNum">搜索最近幅数</param>
        /// <param name="maxFrameInterval"></param>
        /// <param name="heatRating"></param>
        /// <param name="neglectHeatRate"></param>
        /// <param name="Separation"></param>
        /// <param name="shiftRange"></param>
        /// <returns></returns>
        public List<HeatBoltAnalystItem> SearchFeaturedBoltsItem(int startIdx, int endIdx, int searchNum, int maxFrameInterval, int heatRating,
                            int neglectHeatRate, int Separation, int shiftRange)
        {
            startIdx = GetIndexFromID(startIdx);
            endIdx = GetIndexFromID(endIdx);
            if (startIdx < 0 || endIdx < 0)
            {
                return null;
            }
            int i = startIdx, ki;
            int j, k;
            List<HeatBoltAnalystItem> result = new List<HeatBoltAnalystItem>();
            for (i = endIdx, ki = 0; i >= startIdx && ki < searchNum; i--, ki++)
            {
                for (j = i - 1, k = 0; (j >= startIdx) && (k < maxFrameInterval); j--, k++)
                {
                    var h1 = Vector<double>.Build.DenseOfArray(GetHeatsByIndex((i), -1, -1));
                    var h2 = Vector<double>.Build.DenseOfArray(GetHeatsByIndex((j), -1, -1));
                    var v1 = Vector<double>.Build.DenseOfArray(GetThicksByIndex((i), -1, -1));
                    var v2 = Vector<double>.Build.DenseOfArray(GetThicksByIndex((j), -1, -1));
                    var h3 = h2 - h1;
                    var v3 = v1 - v2;
                    var cnt = h3.Count;
                    int t;
                    for (t = 0; t < cnt; t++)
                    {
                        if (Math.Abs(h3[t]) <= neglectHeatRate)
                        {
                            h3[t] = 0;
                        }
                    }
                    if (Separation >= cnt || Separation < 0) Separation = 5;
                    var h4 = h3.Concat(h3).ToArray();
                    t = Separation + shiftRange;
                    var v4 = v3.Concat(v3).Concat(v3).ToArray();
                    DataModel dm = new DataModel(heatRating, Separation, cnt);
                    var h4cnt = h4.Count();
                    for (int ii = 0; ii < h4cnt; ii++)
                    {
                        if (dm.AddOneData((int)h4[ii]) < 0) break;
                    }
                    var tmpr = dm.GetResult();
                    if (tmpr.Count() == 0) continue;
                    h4 = EffectFilter(h3.ToArray());
                    h4 = h4.Concat(h4).Concat(h4).ToArray();
                    foreach (var item in tmpr)
                    {
                        item.FrameIdx1 = _ids[i];
                        item.FrameIdx2 = _ids[j];
                        int from, to;
                        from = item.StartBolt - Separation;
                        to = item.EndBolt + Separation;
                        // 修正from，to的值
                        if (from > to) to += cnt;
                        if (from < 0)
                        {
                            from += cnt;
                            to += cnt;
                        }

                        double maxcor = double.MinValue;
                        int maxshift = 0;
                        for (int jj = -shiftRange; jj <= shiftRange; jj++)
                        {
                            double tmpcor = CalculateCorrel(h4.Skip(from).Take(to - from + 1).ToArray(),
                                                     v4.Skip(from + cnt + jj).Take(to - from + 1).ToArray());
                            if (maxcor < tmpcor)
                            {
                                maxcor = tmpcor;
                                maxshift = jj;
                            }
                        }
                        item.DisPos = maxshift;
                        item.Similarity = maxcor;
                    }
                    if (tmpr.Count > 0)
                    {
                        result.AddRange(tmpr);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 过滤挑选出来的加热特征数据，主要是根据相关性大小选择。
        /// </summary>
        /// <param name="items"></param>
        /// <param name="similarityThredhold">相关性大小的阈值</param>
        /// <returns></returns>
        public List<HeatBoltAnalystItem> FilterFeaturedBoltsItem(List<HeatBoltAnalystItem> items, double similarityThredhold)
        {
            if (items is null) return null;
            List<HeatBoltAnalystItem> tmp = new List<HeatBoltAnalystItem>();
            foreach (HeatBoltAnalystItem i in items)
            {
                bool deal = false;
                foreach (HeatBoltAnalystItem j in tmp)
                {
                    if ((i.StartBolt == j.StartBolt) && (i.EndBolt == j.EndBolt))
                    {
                        deal = true;
                        if (j.Similarity < i.Similarity)
                        {
                            j.Clone(i);
                        }
                    }
                }
                if (!deal && i.Similarity >= similarityThredhold)
                {
                    var j = new HeatBoltAnalystItem();
                    j.Clone(i);
                    tmp.Add(j);
                }
            }
            return tmp;
        }

        /// <summary>
        /// 搜索风环偏转和旋转角度的最佳数值。
        /// </summary>
        /// <param name="startIdx"></param>
        /// <param name="endIdx"></param>
        /// <param name="searchNum">搜索最近幅数</param>
        /// <param name="result"></param>
        /// <param name="report"></param>
        /// <param name="runcomplete"></param>
        /// <param name="midbolt"></param>
        /// <param name="searchRange"></param>
        /// <param name="SearchRotAngle"></param>
        /// <param name="isAsync">是否同步</param>
        public void StartSearchMaxSimilarity(int startIdx, int endIdx, int searchNum, int frameInterval, MaxSimilarityResult result,
                        ProgressChangedEventHandler report,
                        RunWorkerCompletedEventHandler runcomplete,
                        int midbolt, double searchRange, bool SearchRotAngle, bool isAsync)
        {
            BackgroundWorker bk = new BackgroundWorker();
            var handler = new DoWorkEventHandler((o, e) =>
            {
                int fmin = GetIndexFromID(startIdx);
                int fmax = GetIndexFromID(endIdx);
                if (fmin < 0 || fmax < 0)
                {
                    result.frameIdx1 = -1;
                    return;
                }
                int i, j, k, ki;
                double maxcorel = -100;
                int theshift = 0;
                int idx1 = fmin, idx2 = fmax;
                double deltaAngle = 0;
                double factor = 1;
                if (midbolt == int.MaxValue)
                {
                    midbolt = _resetBolts[fmax];
                }
                if (frameInterval < 0)
                {
                    frameInterval = int.MaxValue;
                }
                for (i = fmax, ki = 0; (i > fmin) && (ki < searchNum); i--, ki++)
                {
                    if ((o as BackgroundWorker).WorkerReportsProgress)
                    {
                        (o as BackgroundWorker).ReportProgress((fmax - i) * 100 / (fmax - fmin));
                    }
                    for (j = i - 1, k = 0; (j >= fmin) && (k < frameInterval); j--, k++)
                    {
                        var tmp = SearchMaxSimilarity(j, i, searchRange, SearchRotAngle);
                        if (maxcorel < tmp.Item2)
                        {
                            maxcorel = tmp.Item2;
                            theshift = tmp.Item1;
                            deltaAngle = tmp.Item3;
                            idx1 = i;
                            idx2 = j;
                        }
                    }
                }
                result.frameIdx1 = _ids[idx1];
                result.frameIdx2 = _ids[idx2];
                result.frameShift = theshift;
                result.similarity = maxcorel;
                result.deltaAngle = deltaAngle;
                var angle = GetRotAngle(_ids[idx1]) + deltaAngle;
                var heat = DataHelper.VectorSub(GetHeats(_ids[idx2], -1, -1), GetHeats(_ids[idx1], -1, -1));
                var t1 = GetThicks(_ids[idx1], int.MinValue, angle);
                var th = DataHelper.VectorSub(t1, GetThicks(_ids[idx2], int.MaxValue, angle));
                result.ThickToHeatFactor = CalculateHeatToThickFactor(heat, th,
                    MathNet.Numerics.Statistics.Statistics.Mean(t1));
            });
            if (report != null)
            {
                bk.WorkerReportsProgress = true;
                bk.ProgressChanged += report;
            }
            if (runcomplete != null)
            {
                bk.RunWorkerCompleted += runcomplete;
            }
            if (isAsync)
            {
                bk.DoWork += handler;
                bk.RunWorkerAsync();
            }
            else
            {
                handler(bk, null);
                runcomplete(bk, null);
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="idxL"></param>
        /// <param name="idxB"></param>
        /// <param name="searchRange"></param>
        /// <param name="SearchRotAngle"></param>
        /// <returns>返回值：Tuple<偏转数，相关系数，旋转角度变化量></returns>
        public Tuple<int,double,double> SearchMaxSimilarity(int idxL,int idxB,double searchRange,bool SearchRotAngle)
        {
            double maxcorel = double.MinValue;
            int theshift = 0;
            double deltaAngle = 0;
            if (idxL > idxB)
            {
                var tmp = idxL;
                idxL = idxB;
                idxB = idxL;
            }
            double angle_B = GetRotAngleByIndex(idxB);
            double angle_L = GetRotAngleByIndex(idxL);
            int rb_B = GetResetBoltByIndex(idxB);
            int rb_L = GetResetBoltByIndex(idxL);
            Vector<double> vthickL = Vector<double>.Build.DenseOfArray(GetThicksByIndex(idxL, rb_B, angle_B));
            Vector<double> vthickB = Vector<double>.Build.DenseOfArray(GetThicksByIndex(idxB, -1, -1));
            Vector<double> v_heatL = Vector<double>.Build.DenseOfArray(GetHeatsByIndex(idxL, -1, -1));
            Vector<double> v_heatB = Vector<double>.Build.DenseOfArray(GetHeatsByIndex(idxB, -1, -1));
            if (!SearchRotAngle)
            {
                double[] corels = CalculateCorrelVector((vthickL - vthickB).ToArray(), (v_heatB - v_heatL).ToArray());
                double max = -100;
                var r = CalculateAirRingShiftFromCorelVector(corels);
                int shift = r.Item1;
                if (maxcorel < r.Item2)
                {
                    maxcorel = max;
                    theshift = shift;
                }
            }
            else
            {
                double range = searchRange / 360.0;
                double di = -range;
                double u_range = 360 / angle_B - 1;
                for (; di < u_range; di += 0.001)
                {
                    var vth = ReSampleForChangeRotAngle((vthickL - vthickB).ToArray(), rb_B, 1 + di);
                    double[] corels = CalculateCorrelVector(vth, (v_heatB - v_heatL).ToArray());
                    var r = CalculateAirRingShiftFromCorelVector(corels);
                    int shift = r.Item1;
                    if (maxcorel < r.Item2)
                    {
                        maxcorel = r.Item2;
                        theshift = shift;
                        deltaAngle = di * angle_B;
                    }
                }
            }
            return new Tuple<int, double, double>(theshift, maxcorel, deltaAngle);
        }

        #region 数据库数据变化后自动运行
        [PropertyChanged.DoNotCheckEquality]
        public List<ThickHeatEvent> AutoDetectEvents { get; set; }

        /// <summary>
        /// 自动运行是否搜索旋转角度
        /// </summary>
        public bool HasRotateAngleSearched { get; set; }
        /// <summary>
        /// 搜索旋转角度的范围，例如15，则在当前度数下搜索正负15度
        /// </summary>
        public double SearchingAngle { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public StateCode State { get; protected set; } = StateCode.STC_NODAT;

        /// <summary>
        /// 根据设定的参数，自动搜索风环偏转和旋转角度，数据更新时调用。
        /// </summary>
        private void AutoSearchForAirRingShift(int newitems)
        {
            if (thickClasses.Count <= 0) return;
            if (thickClasses[0].to - thickClasses[0].from < 2) return;
            List<ThickHeatEvent> tmp = new List<ThickHeatEvent>();
            //AutoDetectEvents.Clear();
            MaxSimilarityResult result = new MaxSimilarityResult()
            {
                frameIdx1 = -1
            };
            StartSearchMaxSimilarity(thickClasses[0].from, thickClasses[0].to, newitems, 5, result,
                null,
                (o, e) =>
                {
                    if (result.frameIdx1 >= 0)
                    {
                        var evt = new ThickHeatEvent()
                        {
                            EvtCode = ThickHeatEventCode.EVC_RingShift,
                            EvtData = result as object
                        };
                        tmp.Add(evt);
                    }
                },
                int.MaxValue, 10, true, false);
            var r = SearchFeaturedBoltsItem(thickClasses[0].from, thickClasses[0].to, newitems, 5, 100, 5, 5, 5);
            if (r != null)
            {
                foreach (HeatBoltAnalystItem it in r)
                {
                    tmp.Add(new ThickHeatEvent()
                    {
                        EvtCode = ThickHeatEventCode.EVC_HeatBoltItem,
                        EvtData = it as object
                    });
                }
            }
            AutoDetectEvents = tmp;
        }

        /// <summary>
        /// 发现有数据更新时的动作：
        /// 1.  获取数据库的新数据
        /// 2.  聚类（第一次选择数据）---> 计算最大加热变化量（第二次选择数据）---> 计算相关系数向量 --->
        ///     计算最大相关系数时的偏移量（AirRingShift）
        /// </summary>
        public void RefreshData()
        {
            IsDataReady = false;
            int newItems = 0;
            // 计算风环偏转量
            if (_dat_times.Count() > 0)
            {
                GetDataFromDB();
                newItems = FetchNewData();
                State = StateCode.STC_UPDATING;
            }
            else
            {
                GetDataFromDB(true);
                //GetDataFromDB();
                FetchData();
                newItems = int.MaxValue;
                State = StateCode.STC_READING;
            }
            if (newItems > 0)
            {
                ClusterOnThickMean();
                AutoSearchForAirRingShift(newItems);
                CalculateAirRingShift();
            }
            IsDataReady = true;
            State = StateCode.STC_IDLE;
        }

        #endregion

        #endregion

        #region 自动更新数据的任务
        private Task taskForDB;
        private void StartDBTask()
        {
            taskForDB = Task.Factory.StartNew(() =>
            {
                while (true)
                {
                    if (BeginUse())
                    {
                        RefreshData();
                        EndUse();
                        Thread.Sleep(1000 * 60);
                    }
                    else
                    {
                        Thread.Sleep(1000 * 2);
                    }
                }
            });
        }
        #endregion

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class DataManager
    {
        public static IThickHeatData GetThickDataInstance()
        {
            //return ThickHeatData4SQLite_V1.Instance;
            return ThickHeatData4SQLite_V2.Instance;
        }
    }
}
